---
title: Champs Electriques - Travail Pratique
author:
- MARTIG Joey
- STEFANOVIC Boris
date: 2022-05-23
geometry: "margin=35mm"
mainfont: DejaVu Sans
fontsize: 10pt
header-includes:
- \usepackage{float}
- \let\origfigure\figure
- \let\endorigfigure\endfigure
- \renewenvironment{figure}[1][2] {\expandafter\origfigure\expandafter[H]} {\endorigfigure}
---



# Introduction

Le champ électrique est un _champ vectoriel_ généré par la présence d'une
charge dans un espace.
Une représentation courante de ce champ vectoriel est le dessin de vecteurs à intervalles réguliers autour de la charge.

Une autre représentation est celle des lignes de champ.
C'est autour de cette dernière que se construit ce projet.

![source: https://commons.wikimedia.org/w/index.php?curid=1044851](E_FieldOnePointCharge.png){width="28%"}\ ![source: https://brilliant.org/wiki/electric-field-lines/](fieldlines.jpg){width="72%"}

## Buts

Le but premier de ce projet est de créer un outil de visualisation de
champs électriques pour en faciliter la compréhension.
Il permet d'observer les effets de différents paramètres
(charge, distance, etc.) sur la "forme" du champ.
On peut facilement imaginer réutiliser cet outil dans un autre projet
de simulation, plus complexe.



# Plan

1. **Théorie** Rappel théorique nécessaire à la compréhension du programme
2. **Méthodologie** Organisation de ce travail
3. **Implémentation** Structure du programme et fonctionnement
4. **Résultats** Résultats obtenus et tests effectués
5. **Conclusion** Ce qu'il faut retenir de ce travail



# Théorie

## Charge électrique

La charge électrique peut être comprise comme le déséquilibre entre le
nombre de protons et d'électrons dans un corps.
Une conséquence de ce fait est l'existance d'une charge élémentaire,
qui ne peut pas être divisée en charge non-nulle d'intensité moindre:
la charge d'un seul électron.
Un corps contenant plus d'électrons que de protons aura une charge
dite négative et un corps contenant plus de protons que d'électrons
aura une charge dite positive.
Tout corps chargé aura tendance à retourner à un état neutre
pour autant qu'il soit à proximité d'une charge opposée
avec laquelle il a la possibilité d'échanger des électrons.
L'unité de la charge électrique est le Coulomb $[C]$.

## Champ électrique

Le champ électrique étant un _champ vectoriel_,
sa représentation exacte nécessiterait, une infinité de vecteurs:
un pour chaque point de l'espace.
Le champ électrique d'une charge en un point de l'espace est l'effet
que la charge aura en ce point.
Celui-ci est représentable sous la forme d'un vecteur unique.
Comme dans le cas d'une force résultante, plusieurs charges
produiront en un point donné une valeur de _champ résultant_,
étant la somme vectorielle des champs de toutes les charges,
en ce point.

Tout objet chargé placé dans un champ électrique subira une force due à ce dernier.
L'intensité de cette force est le produit du champ à la position de l'objet
et de la charge de l'objet.
L'expression du champ électrique $\overrightarrow{E}$
d'une charge $Q$ placée à la position $\overrightarrow{p}$,
au point $\overrightarrow{x}$
est la suivante:

$$ \overrightarrow{E_x} = K * \frac{Q}{{|\overrightarrow{x} - \overrightarrow{p}|}^2} * \frac{\overrightarrow{x} - \overrightarrow{p}}{|\overrightarrow{x} - \overrightarrow{p}|} $$



# Méthodologie

## Espace de simulation

Contrairement à la réalité, nous évoluons dans un espace en 2 dimensions.
Celui-ci est limité à un carré de [0, 1]x[0, 1],
ce qui signifie que tout objet dépassant cette limite
ne sera pas calculé.
Cet espace contiendra les charges ainsi que les lignes de champ.
Cela nous permet de borner le nombre de calculs réalisés.
Pour afficher les objets de cet espace, nous représentons
les positions de notre carré comme des proportions de la
taille de l'écran.

## Dessin des lignes de champ

Une ligne de champ nous permet de visualiser le champ électrique
généré par les charges.
Dans notre cas, nous prenons une position aléatoire dans notre espace,
suffisamment éloignée de toute charge, et calculons
la valeur du champ électrique en ce point.
Cela nous donne une direction, que nous suivons dans les deux sens
pour dessiner la ligne, jusqu'à atteindre une charge ou
sortir du carré.
Le résultat de ce processus nous donne une ligne de champ unique.
Il suffit de répéter ces étapes jusqu'à ce que l'on juge le nombre
de lignes suffisant.



# Implémentation

Le programme peut se décomposer en plusieurs sections. La section d'affichage primaire, qui permet d'afficher des lignes ou des cercles, qui seront utilisés plus tard pour afficher des résultats plus complexes. La section de calculs physique, qui permet de calculer les champs à une certaine position dans notre espace. Puis finalement les fonctions qui se servent des deux sections précédentes afin d'afficher les charges et les lignes de champ.

## Dessin de formes géométriques

Celui-ci permettant de tester et d'utiliser toutes les autres fonctionnalités
du projet, il paraissait sage de commencer par implémenter des
fonctions d'affichage de lignes et de cercles.

```c
void gfx_draw_line(struct gfx_context_t *ctxt,
					coordinates_t p0,
					coordinates_t p1,
					uint32_t color);

void gfx_draw_circle(struct gfx_context_t *ctxt,
					coordinates_t c,
					uint32_t r,
					uint32_t color);
```

## Calcul de grandeurs physiques

Ensuite, nous avons écrit les fonctions de calcul de champ électrique
en un point.

$$ E* \frac{\overrightarrow{qP}}{||\overrightarrow{qP}||} $$

```c
bool compute_e(charge_t c, vec2 p, double eps, vec2 *e);
```

$$ Ei * \frac{\overrightarrow{qiP}}{||\overrightarrow{qiP}||} $$
```c
bool compute_total_normalized_e(charge_t *charges,
								int num_charges,
								vec2 p,
								double eps,
								vec2 *e);
```

## Affichage des charges et des lignes de champ

Finalement, nous avons implémenté l'affichage des charges et d'une ligne de champ
passant par un point donné.

```c
static void draw_charges(struct gfx_context_t *context,
							charge_t *charges,
							int num_charges,
							double x0,
							double x1,
							double y0,
							double y1);

static bool draw_field_line(struct gfx_context_t *ctxt,
							charge_t *charges,
							int num_charges,
							double dx,
							vec2 pos0,
							double x0,
							double x1,
							double y0,
							double y1);
```

## Logique principale de haut niveau

Dans un premier temps, nous créons des charges et les
positionnons dans l'espace de simulation.
Nous dessinons ensuite les lignes de champ dans cet espace,
ce qui nous permet de visualiser les champs électriques.

```c
	charge_t charges[NC];
	for (int i = 0; i < NC; ++i)
		charges[i] = charge_create(
			i % 2 == 0 ? rand_one() : -rand_one(),
			vec2_create(rand_one(), rand_one())
		);
	struct gfx_context_t *ctxt = gfx_create("elec", WID, HEI);
	draw_everything(ctxt, charges, NC, NLINES, DX, 0.0, 1.0, 0.0, 1.0);
	gfx_present(ctxt);
	while (gfx_keypressed() != SDLK_ESCAPE) {
		gfx_present(ctxt);
	}
	gfx_destroy(ctxt);
```



# Résultats

Voici un aperçu de résultats que nous avons observés avec
notre programme.
Notons que les charges électriques sont de signes opposés.
On voit très rapidement que les lignes de champ relient les charges.
Cela est effectivement le résultat attendu dans ce cas de figure.
Maintenant, si nous augmentons la valeur absolue d'une des charges,
on remarque que celle-ci a un impact plus important sur le champ
électrique total que l'autre charge.

![](2_charges.png){width="33%"}\ ![](2_charges_+Intensity.png){width="33%"}\ ![](2_charges_-Intensity.png){width="33%"}

La série de tests s'est poursuivie par la vérification du
comportement du modèle lorsque l'on ajoutait un nombre de charges
plus grand que 2 dans l'espace.
Le résultat affiché semble cohérent.

![3 charges](3_charges.png){width="50%"}

En plus de ces tests s'appliquant au modèle physique,
nous avons une batterie de tests permettant de se convaincre de façon
non-exhaustive que les différentes fonctions aient un comportement adéquat.
Dans le cas des fonctions de calcul de grandeurs physiques,
nous avons réalisé des tests unitaires.
Pour l'affichage, le test est visuel et nécessite un bref contrôle humain.
Le rôle de ces tests est de former un premier filtre face aux bugs éventuels.

![draw test](draw_test.png){width="35%"}



# Conclusion

Au cours de ce projet, nous avons pu observer la "forme" du champ électrique dans un espace contenant plusieurs charges. Cela nous permettrait de visualiser, par exemple, la direction du vecteur accélération d'une charge mobile qui se retrouverait dans cet environnement. Les résultats obtenus sont satisfaisants par rapport aux résultats attendus. Surtout lorsque plus de deux charges sont présentent.

Le fonctionnement du programme correspond à la théorie fournie et satisfait donc son but principal étant de permettre la visualisation des champs électriques. Il a été difficile dans le temps imparti de réaliser le programme ainsi que des tests, mais nous avons tout de même réussi à réaliser le travail, ainsi que des tests des fonctions les plus utilisées.

Afin d'améliorer le projet, la première étape serait d'y ajouter les tests unitaires manquants. Pour aller plus loin, il serait envisageable de le réaliser de manière dynamique. Cela demanderait un gros travail d'optimisation, pour obtenir suffisamment de frame par seconde ainsi que l'ajout de déplacement des charges. Ajouter des intéractions serait ludique et permettrait de comprendre plus facilement les lignes de champ. Ce serait faisable en laissant l'utilisateur ajouter des lignes de champ manuellement ou en modifiant en direct les charges.
