/**
 * @file draw.c
 * @author Boris Stefanovic, Joey Martig
 * @brief Contient les méthodes d'affichage de lignes et de cercles.
 * @date 23.05.2022
 */

#include <stdbool.h>
#include "../utils/gfx/gfx.h"
#include "../utils/utils.h"


void gfx_draw_line(struct gfx_context_t *ctxt, coordinates_t p0, coordinates_t p1, uint32_t color) {
	int dx = abs((int) p1.column - (int) p0.column);
	int sx = p0.column < p1.column ? 1 : -1;
	int dy = -abs((int) p1.row - (int) p0.row);
	int sy = p0.row < p1.row ? 1 : -1;
	int error = dx + dy;

	while (true) {
		gfx_putpixel(ctxt, p0.column, p0.row, color);

		if (p0.column == p1.column && p0.row == p1.row)
			break;

		int e2 = 2 * error;

		if (e2 >= dy) {
			if (p0.column == p1.column)
				break;
			error = error + dy;
			p0.column = p0.column + sx;
		}

		if (e2 <= dx) {
			if (p0.row == p1.row)
				break;
			error = error + dx;
			p0.row = p0.row + sy;
		}
	}
}


void gfx_draw_circle(struct gfx_context_t *ctxt, coordinates_t c, uint32_t r, uint32_t color) {
	int x = 0;
	int y = r;
	int d = r - 1;
	while (y >= x) {
		gfx_putpixel(ctxt, c.column + x, c.row + y, color);
		gfx_putpixel(ctxt, c.column + y, c.row + x, color);
		gfx_putpixel(ctxt, c.column - x, c.row + y, color);
		gfx_putpixel(ctxt, c.column - y, c.row + x, color);
		gfx_putpixel(ctxt, c.column + x, c.row - y, color);
		gfx_putpixel(ctxt, c.column + y, c.row - x, color);
		gfx_putpixel(ctxt, c.column - x, c.row - y, color);
		gfx_putpixel(ctxt, c.column - y, c.row - x, color);
		if (d >= 2 * x) {
			d = d - 2 * x - 1;
			x++;
		} else if (d < 2 * ((int) r - y)) {
			d = d + 2 * y - 1;
			y--;
		} else {
			d = d + 2 * (y - x - 1);
			y--;
			x++;
		}
	}
}
